package api

import (
	"sku3dweb/db/model"
	"sku3dweb/db/repo"
	"time"

	"github.com/gin-gonic/gin"
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/bson/primitive"
)

func FassiApi(r *GinRouter) {
	r.POSTJWT("/image/create", createImg)
	r.GETJWT("/image/fassi/list", SearchByImg)
	r.GETJWT("/image/list", SearchByFields)
	r.POSTJWT("/image/delete/:id", DeleteImage)
}

func DeleteImage(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	id := c.Param("id")
	err := RomoveFassiImage(id)
	if err != nil {
		return nil, err
	}
	return repo.RepoDeleteDoc(apictx.CreateRepoCtx(), repo.CollectionMatImages, id)
}

func SearchByFields(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	page, size, query := UtilQueryPageSize(c)

	pageOption := &repo.PageSearchOptions{
		CollectName: repo.CollectionMatImages,
		Page:        page,
		Size:        size,
		Query:       query,
		Project:     []string{"nameCn", "createTime", "thumbnail", "price", "from"},
		// Sort: bson.M{"_id"},
	}
	return repo.RepoPageSearch(apictx.CreateRepoCtx(), pageOption)
}

func createImg(c *gin.Context, apictx *ApiSession) (interface{}, error) {

	data := &model.MatImage{}
	err := c.ShouldBindJSON(data)
	if err != nil {
		return nil, err
	}
	data.CreateTime = time.Now()
	data.UpdateTime = time.Now()
	data.UserId, _ = primitive.ObjectIDFromHex(apictx.User.ID)

	idstr, err := repo.RepoAddDoc(apictx.CreateRepoCtx(), repo.CollectionMatImages, data)
	if err != nil {
		return nil, err
	}
	imgId, _ := primitive.ObjectIDFromHex(idstr)

	return imgId, AddFassiImage(imgId, data.RawImage.Url)
}

func SearchByImg(c *gin.Context, apictx *ApiSession) (interface{}, error) {

	data := &struct {
		Url string
	}{}
	err := c.ShouldBindJSON(data)
	if err != nil || len(data.Url) < 1 {
		return nil, NewError("参数错误")
	}

	images, err := QueryFassiImage(data.Url)
	if err != nil {
		return nil, err
	}

	var list = []map[string]interface{}{}
	if len(images) < 1 {
		return list, nil
	}

	var ids = []primitive.ObjectID{}
	for _, item := range images {
		ids = append(ids, item.Id)

	}
	param := &repo.DocSearchOptions{
		CollectName: repo.CollectionMatImages,
		Query:       repo.Map{"_id": bson.M{"$in": ids}},
	}
	err = repo.RepoSeachDocs(apictx.CreateRepoCtx(), param, &list)
	if err != nil {
		return nil, err
	}

	var ImageMap = map[primitive.ObjectID]map[string]interface{}{}
	for _, item := range list {
		ImageMap[item["_id"].(primitive.ObjectID)] = item
	}

	var out = []interface{}{}
	for _, item := range images {
		value := ImageMap[item.Id]
		value["score"] = item.Score
		out = append(out, value)
	}
	return out, nil
}
