package api

import (
	"errors"
	"sku3dweb/db/model"
	"sku3dweb/db/repo"
	"sku3dweb/log"
	"time"

	"github.com/gin-gonic/gin"
	"go.mongodb.org/mongo-driver/bson/primitive"
)

// CategoryCreate 创建分类
func CategoryCreate(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	var category model.ImCategory
	err := c.ShouldBindJSON(&category)
	if err != nil {
		return nil, err
	}
	category.TypeId = -1
	category.CreateTime = time.Now()
	category.UpdateTime = time.Now()
	return repo.RepoAddDoc(apictx.CreateRepoCtx(), repo.CollectionImCategory, &category)
}

// CategoryDelete 删除分类
func DeleteCategory(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	_id := c.Param("id")
	id, _ := primitive.ObjectIDFromHex(_id)
	if id.IsZero() {
		return nil, errors.New("id错误")
	}
	return repo.RepoDeleteDoc(apictx.CreateRepoCtx(), repo.CollectionImCategory, _id)
}

// CategoryList 获取分类列表
func CategoryList(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	page, size, query := UtilQueryPageSize(c)
	return repo.RepoPageSearch(apictx.CreateRepoCtx(), &repo.PageSearchOptions{
		CollectName: repo.CollectionImCategory,
		Page:        page,
		Size:        size,
		Query:       query,
	})
}

// CategoryUpdate 更新分类
func UpdateCategory(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	category := &model.ImCategory{}
	err := c.ShouldBindJSON(&category)
	if err != nil {
		log.Error(err)
		return nil, err
	}
	if category.Id.IsZero() {
		return nil, errors.New("id错误")
	}
	return repo.RepoUpdateSetDoc(apictx.CreateRepoCtx(), repo.CollectionImCategory, category.Id.Hex(), category)
}
