package api

import (
	"fmt"
	"sku3dweb/db/model"
	"sku3dweb/db/repo"

	"github.com/gin-gonic/gin"
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/bson/primitive"
)

func RegImageMat(router *GinRouter) {

	pageSearchFn := func(c *gin.Context, apictx *ApiSession) (interface{}, error) {
		page, size, query := UtilQueryPageSize(c)

		scope := c.Param("scope")
		coll := UtilGetImgMatCollection(scope)

		pageOption := &repo.PageSearchOptions{
			CollectName: coll,
			Page:        page,
			Size:        size,
			Query:       query,
			Project:     []string{"name", "thumbnail", "cusNum", "createTime", "colorCards", "image", "categories"},
			Sort:        bson.M{"createTime": -1},
		}
		pageResult, err := repo.RepoPageSearch(apictx.CreateRepoCtx(), pageOption)
		if err != nil {
			return nil, err
		}
		for _, v := range pageResult.List {
			if v["userId"] != nil {
				userId := v["userId"].(primitive.ObjectID).Hex()
				u, _ := repo.RedisGetUserById(apictx.CreateRepoCtx(), userId)
				if u != nil {
					v["user"] = u
				}
			}
		}
		return pageResult, nil
	}

	router.GETJWT("/imgmat/list/:scope", pageSearchFn)

	router.GET("/imgmat/detail/:id", func(c *gin.Context, apictx *ApiSession) (interface{}, error) {
		id := c.Param("id")
		if len(id) < 1 {
			return nil, NewError("参数不能为空")
		}
		scope := c.Query("scope")
		out := &model.ImageMat{}
		uid, _ := primitive.ObjectIDFromHex(id)
		Collection := UtilGetImgMatCollection(scope)
		ok, err := repo.RepoSeachDoc(apictx.CreateRepoCtx(), &repo.DocSearchOptions{
			CollectName: Collection,
			Query:       repo.Map{"_id": uid},
			Project:     []string{"name", "image", "thumbnail", "category", "colorCards", "categories"},
		}, out)

		if err != nil {
			return nil, err
		}
		if !ok {
			return nil, NewError("没有查询到数据")
		}
		return out, nil
	})

	router.POSTJWT("/imgmat/update/:scope", func(c *gin.Context, apictx *ApiSession) (interface{}, error) {
		fabric := &model.ImageMat{}
		err := c.ShouldBindJSON(fabric)
		if err != nil {
			fmt.Println(err)
			return nil, NewError("参数解析错误")
		}

		collection := UtilGetImgMatCollection(c.Param("scope"))
		if fabric.Id == primitive.NilObjectID {
			return nil, NewError("Id不能为空!")
		}
		id := fabric.Id.Hex()

		out, err := repo.RepoUpdateSetDoc(apictx.CreateRepoCtx(), collection, id, fabric)

		if err != nil {
			return nil, err
		}
		if out.MatchedCount != 1 {
			return nil, NewError("文件不存在!")
		}
		return true, nil
	})
}
