package api

import (
	"sku3dweb/db/model"
	"sku3dweb/db/repo"
	"time"

	"github.com/gin-gonic/gin"
	"go.mongodb.org/mongo-driver/bson"
)

func CreatePermsRouter(router *GinRouter) {

	//分类操作
	CreateCRUD(router, "/perms", &CRUDOption{
		Collection: repo.CollectionPerms,
		NewModel: func(c *gin.Context) interface{} {
			cat := &model.Perms{}
			c.ShouldBindJSON(cat)
			return cat
		},
		EmtyModel: func(*gin.Context) interface{} {
			return &model.Category{}
		},
		SearchProject: []string{"group", "children"},
	})

	//更新个人设计权限
	router.POST("/feature/person", func(c *gin.Context, apictx *ApiSession) (interface{}, error) {
		entity := &model.SystemPersonFeatures{}
		c.ShouldBindJSON(entity)
		entity.Type = model.Feature_Person
		entity.UpdateTime = time.Now()
		return repo.RepoUpsertSetDoc(apictx.CreateRepoCtx(), repo.CollectionSettings, bson.M{"type": model.Feature_Person}, entity)
	})

	router.GET("/feature/person", func(c *gin.Context, apictx *ApiSession) (interface{}, error) {
		entity := &model.SystemPersonFeatures{}
		_, err := repo.RepoSeachDoc(apictx.CreateRepoCtx(), &repo.DocSearchOptions{
			CollectName: repo.CollectionSettings,
			Query:       repo.Map{"type": model.Feature_Person},
		}, entity)
		if err != nil {
			return nil, err
		}
		return entity, nil
	})

	//更新企业功能设计
	router.POST("/feature/company", func(c *gin.Context, apictx *ApiSession) (interface{}, error) {
		entity := &model.SystemCompanyFeatures{}
		c.ShouldBindJSON(entity)
		entity.Type = model.Feature_Company
		entity.UpdateTime = time.Now()
		return repo.RepoUpsertSetDoc(apictx.CreateRepoCtx(), repo.CollectionSettings, bson.M{"type": model.Feature_Company}, entity)
	})
	router.GET("/feature/company", func(c *gin.Context, apictx *ApiSession) (interface{}, error) {
		entity := &model.SystemCompanyFeatures{}
		_, err := repo.RepoSeachDoc(apictx.CreateRepoCtx(), &repo.DocSearchOptions{
			CollectName: repo.CollectionSettings,
			Query:       repo.Map{"type": model.Feature_Company},
		}, entity)
		if err != nil {
			return nil, err
		}
		return entity, nil
	})

	//更新其他功能设计
	router.POST("/feature/other", func(c *gin.Context, apictx *ApiSession) (interface{}, error) {
		entity := &model.SystemOtherFeatures{}
		c.ShouldBindJSON(entity)
		entity.Type = model.Feature_Other
		entity.UpdateTime = time.Now()
		return repo.RepoUpsertSetDoc(apictx.CreateRepoCtx(), repo.CollectionSettings, bson.M{"type": model.Feature_Other}, entity)
	})
	router.GET("/feature/other", func(c *gin.Context, apictx *ApiSession) (interface{}, error) {
		entity := &model.SystemOtherFeatures{}
		_, err := repo.RepoSeachDoc(apictx.CreateRepoCtx(), &repo.DocSearchOptions{
			CollectName: repo.CollectionSettings,
			Query:       repo.Map{"type": model.Feature_Other},
		}, entity)
		if err != nil {
			return nil, err
		}
		return entity, nil
	})

	router.GET("/features", func(c *gin.Context, apictx *ApiSession) (interface{}, error) {
		var entities []*model.SystemOtherFeatures = []*model.SystemOtherFeatures{}
		err := repo.RepoSeachDocs(apictx.CreateRepoCtx(), &repo.DocSearchOptions{
			CollectName: repo.CollectionSettings,
			Query:       repo.Map{"type": bson.M{"$in": []string{model.Feature_Other, model.Feature_Company, model.Feature_Person}}},
		}, &entities)

		if err != nil {
			return nil, err
		}

		// out := map[string][]string{}
		// for _, item := range entities {
		// 	perms := []string{}
		// 	if item.Value != nil && item.Value.Features != nil {
		// 		for _, group := range item.Value.Features {
		// 			if *group.Enable && len(group.Children) > 0 {
		// 				for _, ite := range group.Children {
		// 					if len(ite.Value) > 0 && *ite.Enable {
		// 						perms = append(perms, ite.Value)
		// 					}
		// 				}
		// 			}
		// 		}
		// 	}
		// 	out[item.Type] = perms
		// }
		return entities, nil
	})
}
