package api

import (
	"sku3dweb/db/model"
	"sku3dweb/db/repo"
	"time"

	"github.com/gin-gonic/gin"
)

func CreateResRouter(router *GinRouter) {

	//创建贴图操作
	CreateCRUD(router, "/texture", &CRUDOption{
		Collection: repo.CollectionTexture,
		NewModel: func(*gin.Context) interface{} {
			return &model.Texture{
				CreateTime: time.Now(),
			}
		},
		EmtyModel: func(*gin.Context) interface{} {
			return &model.Texture{}
		},
		SearchProject: []string{"image", "createTime"},
	})

	//创建饰品
	CreateCRUD(router, "/decorate", &CRUDOption{
		Collection: repo.CollectionDecorates,
		NewModel: func(c *gin.Context) interface{} {
			body := &struct {
				Category string
				Keyword  string
			}{}
			c.ShouldBindJSON(&body)

			return &model.Mesh{
				Name:       "饰品名字",
				Category:   body.Category,
				CreateTime: time.Now(),
				Geometries: []string{},
			}
		},
		EmtyModel: func(*gin.Context) interface{} {
			return &model.Mesh{}
		},
		SearchProject: []string{"name", "createTime", "geometries", "thumbnail", "osgjs", "category", "osgjsbin", "file", "geometries"},
		DetailProject: []string{"name", "materials", "osgjs"},
	})

	//env3d
	CreateCRUD(router, "/env3d", &CRUDOption{
		Collection: repo.CollectionEvn3d,
		NewModel: func(c *gin.Context) interface{} {
			body := &struct {
				Name      string
				Thumbnail *model.OssType
				Config    *model.Evn3dHdrConf
				Hdr       *model.OssType
			}{}
			c.ShouldBindJSON(&body)

			return &model.Env3d{
				Name:      body.Name,
				Thumbnail: body.Thumbnail,
				Config:    body.Config,
				HDR:       body.Hdr,
				Background: &model.Evn3dBackground{
					Type:  1,
					Color: &model.Vect3{0.749, 0.749, 0.749},
				},
				Options: &model.Env3dOption{
					Rotation: 0,
					Exposure: 1.3,
				},
				ToneMap: &model.ToneMap{
					Method:     1,
					Exposure:   1,
					Contrast:   0,
					Brightness: 0,
					Saturation: 1,
				},
				CreateTime: time.Now(),
			}
		},
		EmtyModel: func(*gin.Context) interface{} {
			return &model.Env3d{}
		},
		SearchProject: []string{"name", "config", "thumbnail", "hdr", "createTime", "options", "background", "toneMap"},
	})

	// //产品模板
	// CreateCRUD(router, "/productTpl", &CRUDOption{
	// 	Collection: repo.CollectionProductTpls,
	// 	NewModel: func(c *gin.Context) interface{} {
	// 		// body := &struct {
	// 		// 	Category string
	// 		// 	Keyword  string
	// 		// }{}
	// 		// c.ShouldBindJSON(&body)
	// 		isPublic := false

	// 		return &model.ProductTpl{
	// 			Name:       "单品模板",
	// 			Components: []*model.ProductTplComponent{},
	// 			IsPublic:   &isPublic,
	// 			Thumbnail:  &model.OssType{Url: "", Size: 0},
	// 			CreateTime: time.Now(),
	// 		}
	// 	},
	// 	EmtyModel: func(*gin.Context) interface{} {
	// 		return &model.ProductTpl{}
	// 	},
	// 	SearchProject: []string{"name", "meshId", "thumbnail", "components", "isPublic", "createTime"},
	// })

	// //纸张(基础材质)
	// CreateCRUD(router, "/basemat", &CRUDOption{
	// 	Collection: repo.CollectionBaseMats,
	// 	NewModel: func(c *gin.Context) interface{} {
	// 		body := &struct {
	// 			Color   string
	// 			Type    string
	// 			Keyword string
	// 		}{}
	// 		c.ShouldBindJSON(&body)

	// 		now := time.Now()
	// 		return &model.BaseMaterial{
	// 			Name:       "纸张",
	// 			Thumbnail:  &model.OssType{Url: "", Size: 0},
	// 			CreateTime: now,
	// 			Color:      body.Color,
	// 			Type:       body.Type,
	// 		}
	// 	},

	// 	EmtyModel: func(*gin.Context) interface{} {
	// 		return &model.BaseMaterial{}
	// 	},
	// 	SearchProject: []string{"name", "thumbnail", "isPublic", "createTime", "color", "type", "matId"},
	// })

	//分类操作
	CreateCRUD(router, "/category", &CRUDOption{
		Collection: repo.CollectionCategory,
		NewModel: func(*gin.Context) interface{} {
			return &model.Category{
				CreateTime: time.Now(),
			}
		},
		EmtyModel: func(*gin.Context) interface{} {
			return &model.Category{}
		},
		SearchProject: []string{"name", "type", "children", "createTime"},
	})
}
