package model

import (
	"time"

	"go.mongodb.org/mongo-driver/bson/primitive"
)

var (
	FREE       = 0
	Enterprise = 1
)

// 会员
type Member struct {
	Id         *primitive.ObjectID `bson:"_id,omitempty" json:"_id,omitempty"`
	CreateTime time.Time           `bson:"createTime,omitempty" json:"createTime,omitempty"`
	UpdateTime time.Time           `bson:"updateTime,omitempty" json:"updateTime,omitempty"`
	Level      *int                `bson:"level,omitempty" json:"level,omitempty"`
	Name       string              `bson:"name,omitempty" json:"name,omitempty"`
	Price      *int                `bson:"price,omitempty" json:"price,omitempty"`
	// 价格描述
	PriceDesc string `bson:"priceDesc,omitempty" json:"priceDesc,omitempty"`
	// 订购席数
	UserNum *int `json:"userNum,omitempty" bson:"userNum,omitempty"`
	// 项目数量
	ProjectNum *int `bson:"projectNum,omitempty" json:"projectNum,omitempty"`
	// 时间 (购买多久)
	Day *int `bson:"day,omitempty" json:"day,omitempty"`
}

// 会员用户
type MemberUser struct {
	Id         *primitive.ObjectID `bson:"_id,omitempty" json:"_id,omitempty"`
	CreateTime time.Time           `bson:"createTime,omitempty" json:"createTime,omitempty"`
	UpdateTime time.Time           `bson:"updateTime,omitempty" json:"updateTime,omitempty"`

	// 会员到期时间
	DeadTime time.Time `bson:"deadTime,omitempty" json:"deadTime,omitempty"`
	// 那种会员 免费版还是会员版
	MemberId *primitive.ObjectID `bson:"memberId,omitempty" json:"memberId,omitempty"`
	// 企业用户id
	ParentId *primitive.ObjectID `bson:"parentId,omitempty" json:"parentId,omitempty"`
	// 订购席数
	UserNum *int `json:"usernum,omitempty" bson:"usernum,omitempty"`

	// 企业名称
	CompanyName string `bson:"companyName,omitempty" json:"companyName,omitempty"`
	Phone       string `bson:"phone,omitempty" json:"phone,omitempty"`

	// 那些子用户可以用
	UserIds []*primitive.ObjectID `bson:"userIds,omitempty" json:"userIds,omitempty"`
}

type MemberUsers struct {
	Id         *primitive.ObjectID `bson:"_id,omitempty" json:"_id,omitempty"`
	CreateTime time.Time           `bson:"createTime,omitempty" json:"createTime,omitempty"`
	UpdateTime time.Time           `bson:"updateTime,omitempty" json:"updateTime,omitempty"`

	// 会员到期时间
	DeadTime time.Time `bson:"deadTime,omitempty" json:"deadTime,omitempty"`
	// 那种会员 免费版还是会员版
	MemberLevelId *primitive.ObjectID `bson:"memberId,omitempty" json:"memberId,omitempty"`

	// 企业用户id
	ParentId *primitive.ObjectID `bson:"parentId,omitempty" json:"parentId,omitempty"`

	Paying bool

	// 订购席数
	UserNum *int `json:"usernum,omitempty" bson:"usernum,omitempty"`

	// 企业名称
	CompanyName string `bson:"companyName,omitempty" json:"companyName,omitempty"`
	Phone       string `bson:"phone,omitempty" json:"phone,omitempty"`

	// 那些子用户可以用
	UserIds []*primitive.ObjectID `bson:"userIds,omitempty" json:"userIds,omitempty"`
}
