package model

import (
	"time"

	"go.mongodb.org/mongo-driver/bson/primitive"
)

//企业账户的角色列表
type FilterTime struct {
	Year    int32
	Seasons []int32
}

//企业账户的角色列表
type FilterCategory struct {
	Category1 string
	Category2 []string
}

type LibFilter struct {
	Values []string `bson:"values,omitempty" json:"values"` //企业类型["设计中心(企业平台)-运动鞋设计部(运动鞋设计部)-小组（设计1小组）-个人"]
	CateId string   `bson:"cateId,omitempty" json:"cateId"` //年限
	IsAll  *bool    `bson:"isAll,omitempty" json:"isAll"`   //性别
}

type MatFilter struct {
	QiyeFrom   []string          //企业类型["设计中心(企业平台)-运动鞋设计部(运动鞋设计部)-小组（设计1小组）-个人"]
	YearSeason []*FilterTime     //年限
	Category   []*FilterCategory //分类
}

type TeamResFilter struct {
	ShoeCategoryIds     []*LibFilter `bson:"shoeCategoryIds,omitempty" json:"shoeCategoryIds"`         //分配的分类列表
	MatCategoryIds      []*LibFilter `bson:"matCategoryIds,omitempty" json:"matCategoryIds"`           //分配的分类列表
	SoleCategoryIds     []*LibFilter `bson:"soleCategoryIds,omitempty" json:"soleCategoryIds"`         //分配的分类列表
	HeelCategoryIds     []*LibFilter `bson:"heelCategoryIds,omitempty" json:"heelCategoryIds"`         //分配的分类列表
	DecorateCategoryIds []*LibFilter `bson:"decorateCategoryIds,omitempty" json:"decorateCategoryIds"` //分配的分类列表
}

type QiyeRole struct {
	Id     primitive.ObjectID `bson:"_id,omitempty" json:"_id"`
	UserId string             `bson:"userId,omitempty" json:"userId"`         //企业主账户id
	Name   string             `bson:"name, omitempty" json:"name,omitempty"`  //角色名字
	Perms  []string           `bson:"perms,omitempty" json:"perms,omitempty"` //权限名字列表
}

type Team struct {
	Id         primitive.ObjectID `bson:"_id,omitempty" json:"_id,omitempty"`
	UserId     string             `bson:"userId,omitempty" json:"userId,omitempty"` //企业主账户id
	Name       string             `bson:"name" json:"name"`                         //角色名字
	UserCount  int32              `bson:"userCount" json:"userCount"`               //用户个数
	CreateTime time.Time          `bson:"createTime,omitempty" json:"createTime,omitempty"`
	Filter     *TeamResFilter     `bson:"filters" json:"filters"`                       //资源名字列表
	LeaderId   string             `bson:"leaderId,omitempty" json:"leaderId,omitempty"` //leaderId
}

type ResCategory struct {
	Id         primitive.ObjectID `bson:"_id,omitempty" json:"_id"`
	Type       string             `bson:"type,omitempty" json:"type"`
	UserId     string             `bson:"userId,omitempty" json:"userId"`
	Parent     string             `bson:"parent,omitempty" json:"parent"`
	Name       string             `bson:"name,omitempty" json:"name"`
	Value      string             `bson:"value,omitempty" json:"value"`
	Level      *int32             `bson:"level,omitempty" json:"level"`
	CreateTime time.Time          `bson:"createTime,omitempty" json:"createTime"`
	OrderId    int64              `bson:"orderId,omitempty" json:"orderId"`
}

type LibCategory struct {
	Id          primitive.ObjectID `bson:"_id,omitempty" json:"_id"`
	Type        string             `bson:"type,omitempty" json:"type"`
	UserId      string             `bson:"userId,omitempty" json:"userId"`
	CategoryIds []string           `bson:"categoryIds,omitempty" json:"categoryIds"`
	CreateTime  time.Time          `bson:"createTime,omitempty" json:"createTime"`
}

type Role struct {
	Id         primitive.ObjectID `bson:"_id,omitempty" json:"_id"`
	UserId     string             `bson:"userId,omitempty" json:"userId"` //主账号Id
	Name       string             `bson:"name,omitempty" json:"name"`
	Perms      []string           `bson:"perms,omitempty" json:"perms"`
	CreateTime time.Time          `bson:"createTime,omitempty" json:"createTime"`
}
